/*
 *  メッセージウインドウクラス
 */
package jp.programmingmat.www.atfuiwnd;

import java.util.ArrayList;
import android.graphics.*;

public class MessageWindow {

	private int mX, mY;
	private int mBorderWidth;
	private int mPageMarkerHeight;

	private int mWindowWidth, mWindowHeight;
	private int mContentMargin;
	
	private Rect mWindowRect, mClientRect, mContentRect;
	private Path mPageMarkerPath;
	
	private Paint mBgPaint, mBorderPaint, mTextPaint, mPageMarkerPaint;
	
	private int mTextHeight;
	private int mPageLines, mLineHeight;
	
	private int mPuttingLine, mPuttingPos;
	
	private int mPageIndex;
	
	private int mProcessStage;
	
	private ArrayList<ArrayList<String>> mPageList;

	private int mProcessCount;
	
	public static final int PROCESS_STAGE_NONE = 0;
	public static final int PROCESS_STAGE_PUTTING = 1;
	public static final int PROCESS_STAGE_PAGE_WAIT = 2;
	public static final int PROCESS_STAGE_PAGE_START = 3;
	public static final int PROCESS_STAGE_PAGE_END = 4;
	public static final int PROCESS_STAGE_PAGE_EXIT = 5;
	public static final int PROCESS_STAGE_MESSAGE_EXIT = 6;
	public static final int PROCESS_STAGE_EXIT = 7;
	
	private boolean mVisible;
	
	public MessageWindow() {
		
		mBgPaint = new Paint();
		
		mBgPaint.setARGB(128, 0, 0, 0);

		mBorderPaint = new Paint();

		mBorderPaint.setARGB(160, 255, 255, 255);
		
		mTextPaint = new Paint();

		mTextPaint.setARGB(255, 255, 255, 255);
	
		mPageMarkerPaint = new Paint();

		mPageMarkerPaint.setARGB(255, 255, 255, 255);
	
		mWindowRect = new Rect();
		mClientRect = new Rect();
		mContentRect = new Rect();

		mPageMarkerPath = new Path();
		
		mBorderWidth = 2;
		mContentMargin = 2;
		
		mTextHeight = (int)(mTextPaint.getFontMetrics().bottom - mTextPaint.getFontMetrics().top);
		mLineHeight = mTextHeight + 2;

		mPageMarkerHeight = 12;
		
		mX = 0;
		mY = 0;
		
		setSize(120, 120);
		setPos(20, 20);

		mProcessCount = 0;
		mVisible = false;
		
	}

	public void setPos(int x, int y) {
		
		mX = x;
		mY = y;

		setWindowRect();
		
	}

	// コンポーネント各種サイズ属性再設定
	private void setWindowRect() {
		
		mWindowRect.left = mX;
		mWindowRect.top = mY;
		mWindowRect.right = mX + mWindowWidth;
		mWindowRect.bottom = mY + mWindowHeight;

		mClientRect.left = mWindowRect.left + mBorderWidth;
		mClientRect.top = mWindowRect.top + mBorderWidth;
		mClientRect.right = mWindowRect.right - mBorderWidth;
		mClientRect.bottom = mWindowRect.bottom - mBorderWidth;

		mContentRect.left = mClientRect.left + mContentMargin;
		mContentRect.top = mClientRect.top + mContentMargin;
		mContentRect.right = mClientRect.right - mContentMargin;
		mContentRect.bottom = mClientRect.bottom - mContentMargin;

		mPageMarkerPath.reset();
		
		mPageMarkerPath.moveTo(mClientRect.centerX() - (mPageMarkerHeight - 2) / 2, mClientRect.bottom - (mPageMarkerHeight - 1));
		mPageMarkerPath.lineTo(mClientRect.centerX(), mClientRect.bottom - 1);
		mPageMarkerPath.lineTo(mClientRect.centerX() + (mPageMarkerHeight - 2) / 2, mClientRect.bottom - (mPageMarkerHeight - 1));
		mPageMarkerPath.close();
		
		mPageLines = (mContentRect.height() - mPageMarkerHeight) / mLineHeight;
		
	}

	public void setSize(int w, int h) {

		mWindowWidth = w;
		mWindowHeight = h;

		setWindowRect();
		
	}
	
	private int getStringWidth(String str) {

		Rect rec = new Rect();
		
		mTextPaint.getTextBounds(str, 0, str.length(), rec);

		return rec.right;
		
	}

	public void setVisible(boolean visible) {
		mVisible = visible;
	}

	// 表示文字列を設定
	public void setText(String text) {
		
		mPageList = new ArrayList<ArrayList<String>>();

		String line = new String();
		
		ArrayList<String> lineList = new ArrayList<String>();

		int pos = 0;
		
		boolean pageFlag;
		
		while (pos < text.length()) {

			pageFlag = false;
			
			while (pos < text.length() && getStringWidth(line + text.charAt(pos)) < mContentRect.width()) {

				// 現在位置に<:br>タグがあれば改行し現在位置をタグの次の文字へ
				if (text.indexOf("<:br>", pos) == pos) {

					pos += 5;

					// 現在行の処理を終了する
					break;

				}

				// 現在位置に<:page>タグがあれば改ページ処理へ
				if (text.indexOf("<:page>", pos) == pos) {

					pos += 7;

					// 改ページフラグセット
					pageFlag = true;

					// 現在行の処理を終了する
					break;

				}

				// 現在行に現在位置の文字を追加
				line = line + text.charAt(pos);

				// 現在位置更新
				pos++;

			}

			// 現在行の文字列を行配列に追加
			lineList.add(line);

			// 改ページフラグがセットされていたら改ページ文字を挿入
			if (pageFlag) {
				lineList.add("\f");
			}

			// 現在行の文字列を初期化
			line = new String();

		}

		// 逐次出力中の文字列位置
		int pagePos = 0;
		int linePos = 0;

		// 行配列をページ配列に格納
		while (linePos < lineList.size()) {

			// 新ページ作成
			mPageList.add(new ArrayList<String>());

			// ページ配列の現在ページに行配列の各行を追加
			for (int i = 0;(i < mPageLines) && (linePos < lineList.size());i++) {

				// 改ページ文字判定
				if (lineList.get(linePos).length() > 0 && lineList.get(linePos).charAt(0) == '\f') {

					// 現在行位置を改ページ文字行の次へ
					linePos++;

					// ページ先頭でなければ改ページ処理実行
					if (i > 0) {

						// 現在ページを終了し新規ページへ
						break;

					}

				} else {

					// ページ配列の現在位置に行配列の文字列をコピー
					mPageList.get(pagePos).add(lineList.get(linePos++));

				}

			}

			pagePos++;

		}

		// 処理状態を逐次出力に
		mProcessStage = MessageWindow.PROCESS_STAGE_PUTTING;

		// 現在ページ位置を最初のページに
		mPageIndex = 0;

		// 現在表示位置初期化
		mPuttingLine = 0;
		mPuttingPos = 5;
			
	}

	public boolean process(SoftPad pad) {

		switch (mProcessStage) {

		// ページ開始
		case MessageWindow.PROCESS_STAGE_PAGE_START:

			// リターンキー解放待機後にページ表示処理開始
			if (pad.getTrigger() == 0) {

				mPuttingLine = 0;
				mPuttingPos = 0;

				mProcessStage = MessageWindow.PROCESS_STAGE_PUTTING;

			}

			break;

		// ページ表示完了
		case MessageWindow.PROCESS_STAGE_PAGE_END:

			// リターンキー解放待機後にページ／メッセージ終了処理へ
			if (pad.getTrigger() == 0) {
				if (mPageIndex < mPageList.size() - 1) {
					mProcessStage = MessageWindow.PROCESS_STAGE_PAGE_EXIT;
				} else {
					mProcessStage = MessageWindow.PROCESS_STAGE_MESSAGE_EXIT;
				}
			}

			break;

		// ページ終了処理
		case MessageWindow.PROCESS_STAGE_PAGE_EXIT:

			// リターンキーが押されていたら、次ページ開始
			if (pad.getTrigger() == SoftPad.PADACTIVE_TRIGGER_A) {

				mPageIndex++;

				mProcessStage = MessageWindow.PROCESS_STAGE_PAGE_START;

			}

			break;

		// メッセージ（全ページ）終了処理
		case MessageWindow.PROCESS_STAGE_MESSAGE_EXIT:

			// リターンキーが押されていたら、終了状態へ
			if (pad.getTrigger() == SoftPad.PADACTIVE_TRIGGER_A) {
				mProcessStage = MessageWindow.PROCESS_STAGE_EXIT;
			}

			break;

		// ページ文字列逐次出力中
		case MessageWindow.PROCESS_STAGE_PUTTING:

			// リターンキーが押されていたらページ全表示
			if (pad.getTrigger() == SoftPad.PADACTIVE_TRIGGER_A) {

				mProcessStage = MessageWindow.PROCESS_STAGE_PAGE_END;

				return false;
				
			}
			
			if (mProcessCount % 5 == 0) {

				// 表示位置更新
				mPuttingPos++;
				
				// 一行の表示終了
				if (mPuttingPos >= mPageList.get(mPageIndex).get(mPuttingLine).length()) {

					// ページ全行の表示完了ならページ終了
					if (mPuttingLine == mPageList.get(mPageIndex).size() - 1) {
						mProcessStage = MessageWindow.PROCESS_STAGE_PAGE_END;
					} else {

						// 次の行へ
						mPuttingLine++;
						mPuttingPos = 0;

					}

				}

			}
			
			break;

		}

		mProcessCount++;

		// 表示終了状態ならtrue
		return mProcessStage == MessageWindow.PROCESS_STAGE_EXIT;
		
		
	}
	public void draw(Canvas cv) {

		if (!mVisible) {
			return;
		}

		// 背景クリア
		cv.drawRect(mWindowRect, mBorderPaint);
		cv.drawRect(mClientRect, mBgPaint);
		
		switch (mProcessStage) {

		// ページ文字列逐次出力中
		case MessageWindow.PROCESS_STAGE_PUTTING:

			// 現在行の前まで一気に出力
			for (int i = 0;i < mPuttingLine;i++) {
				cv.drawText(mPageList.get(mPageIndex).get(i), mContentRect.left, mTextHeight + mContentRect.top + i * mLineHeight, mTextPaint);
			}

			// 現在表示中の行の文字列を表示位置の文字まで描画
			cv.drawText(mPageList.get(mPageIndex).get(mPuttingLine).substring(0, mPuttingPos), mContentRect.left, mTextHeight + mContentRect.top + mPuttingLine * mLineHeight, mTextPaint);

			break;

		// ページ全体表示中
		case MessageWindow.PROCESS_STAGE_PAGE_END:
		case MessageWindow.PROCESS_STAGE_PAGE_EXIT:
		case MessageWindow.PROCESS_STAGE_MESSAGE_EXIT:

			// 現在ページ内にある行配列の文字列を描画
			for (int i = 0;i < mPageList.get(mPageIndex).size();i++) {
				cv.drawText(mPageList.get(mPageIndex).get(i), mContentRect.left, mTextHeight + mContentRect.top + i * mLineHeight, mTextPaint);
			}

			break;

		}
		
		// 次ページ待機中なら、次ページマーカーを点滅表示
		if (mProcessStage == MessageWindow.PROCESS_STAGE_PAGE_EXIT && (mProcessCount % 20) < 10) {
			cv.drawPath(mPageMarkerPath, mPageMarkerPaint);
		}

	}
	
}