/*
 *  選択ダイアログクラス
 */
package jp.programmingmat.www.atfuiwnd;

import android.graphics.*;

public class SelectDialog {

	private int mX, mY;
	private int mBorderWidth;
	private int mSelectMarkerWidth;

	private int mContentMargin;
	
	private Rect mWindowRect, mClientRect, mContentRect, mStringItemRect;
	
	private Paint mBgPaint, mBorderPaint, mTextPaint, mSelectMarkerPaint;
	
	private int mTextHeight;
	private int mLineHeight;
	
	private int mProcessStage;

	private String[] mItemList;
	private Path[] mSelectMarkerPathList;

	private int mProcessCount;
	private int mCursorWait;
	
	public static final int PROCESS_STAGE_NONE = 0;
	public static final int PROCESS_STAGE_START = 1;
	public static final int PROCESS_STAGE_SHOW = 2;
	public static final int PROCESS_STAGE_SELECTED = 3;
	public static final int PROCESS_STAGE_EXIT = 4;

	private int mSelectedIndex;
	private boolean mVisible;
	
	public SelectDialog() {
		
		mBgPaint = new Paint();
		
		mBgPaint.setARGB(128, 0, 0, 0);

		mBorderPaint = new Paint();

		mBorderPaint.setARGB(160, 255, 255, 255);
		
		mTextPaint = new Paint();

		mTextPaint.setARGB(255, 255, 255, 255);
	
		mSelectMarkerPaint = new Paint();

		mSelectMarkerPaint.setARGB(255, 255, 255, 255);
	
		mWindowRect = new Rect();
		mClientRect = new Rect();
		mContentRect = new Rect();
		mStringItemRect = new Rect();
		
		mBorderWidth = 2;
		mContentMargin = 2;
		
		mTextHeight = (int)(mTextPaint.getFontMetrics().bottom - mTextPaint.getFontMetrics().top);
		mLineHeight = mTextHeight + 2;

		mSelectMarkerWidth = 12;

		mX = 0;
		mY = 0;

		setPos(20, 20);

		mProcessCount = 0;
		mCursorWait = 0;

		mVisible = false;
		
	}

	public void setPos(int x, int y) {

		mX = x;
		mY = y;

		setWindowRect();

	}

	public void setVisible(boolean visible) {
		mVisible = visible;
	}

	// コンポーネント各種サイズ属性再設定
	private void setWindowRect() {

		if (mItemList == null || mItemList.length < 1) {
			return;
		}

		int itemWidth = 0;

		for (int i = 0;i < mItemList.length;i++) {
			if (getStringWidth(mItemList[i]) > itemWidth) {
				itemWidth = getStringWidth(mItemList[i]);
			}
		}

		mWindowRect.left = mX;
		mWindowRect.top = mY;
		mWindowRect.right = mX + mSelectMarkerWidth + itemWidth + mContentMargin * 2 + mBorderWidth * 2;
		mWindowRect.bottom = mY + mItemList.length * mLineHeight + mContentMargin * 2 + mBorderWidth * 2;

		mClientRect.left = mWindowRect.left + mBorderWidth;
		mClientRect.top = mWindowRect.top + mBorderWidth;
		mClientRect.right = mWindowRect.right - mBorderWidth;
		mClientRect.bottom = mWindowRect.bottom - mBorderWidth;

		mContentRect.left = mClientRect.left + mContentMargin;
		mContentRect.top = mClientRect.top + mContentMargin;
		mContentRect.right = mClientRect.right - mContentMargin;
		mContentRect.bottom = mClientRect.bottom - mContentMargin;

		mStringItemRect.left = mContentRect.left + mSelectMarkerWidth;
		mStringItemRect.top = mContentRect.top;
		mStringItemRect.right = mContentRect.right;
		mStringItemRect.bottom = mContentRect.bottom;

		mSelectMarkerPathList = new Path[mItemList.length];

		for (int i = 0;i < mItemList.length;i++) {

			Path path = new Path();

			int lineCenter = mContentRect.top + i * mLineHeight + (int)(mLineHeight / 2);
			
			path.moveTo(mContentRect.left + 1, lineCenter - (mSelectMarkerWidth / 2) + 1);
			path.lineTo(mContentRect.left + mSelectMarkerWidth - 1, lineCenter);
			path.lineTo(mContentRect.left + 1, lineCenter + (mSelectMarkerWidth / 2) - 1);

			path.close();
			
			mSelectMarkerPathList[i] = path;
			
		}

	}

	private int getStringWidth(String str) {

		if (str == null || str.length() < 1) {
			return 0;
		}

		Rect rec = new Rect();

		mTextPaint.getTextBounds(str, 0, str.length(), rec);

		return rec.right;

	}

	public void setItems(String items[]) {

		int itemWidth = 0;

		mItemList = new String[items.length];

		for (int i = 0;i < items.length;i++) {

			if (getStringWidth(items[i]) > itemWidth) {
				itemWidth = getStringWidth(items[i]);
			}

			mItemList[i] = new String(items[i]);

		}

		setWindowRect();

		mProcessStage = SelectDialog.PROCESS_STAGE_START;
		
	}

	public boolean process(SoftPad pad) {

		switch (mProcessStage) {

		// ページ開始
		case SelectDialog.PROCESS_STAGE_START:

			// リターンキー解放待機後にページ表示処理開始
			if (pad.getTrigger() == 0) {

				if (mItemList == null || mItemList.length < 1) {
					mProcessStage = SelectDialog.PROCESS_STAGE_SELECTED;
				} else {
					mProcessStage = SelectDialog.PROCESS_STAGE_SHOW;
				}
			
			}

			break;

		// 選択中
		case SelectDialog.PROCESS_STAGE_SHOW:

			// リターンキーが押されていたら、選択状態へ
			if (pad.getTrigger() == SoftPad.PADACTIVE_TRIGGER_A) {
				mProcessStage = SelectDialog.PROCESS_STAGE_SELECTED;
			} else {
				
				if (mCursorWait > 0) {
					mCursorWait--;
				} else {
					switch (pad.getStick()) {

					case SoftPad.PADACTIVE_UP:
					
						mSelectedIndex--;
						mCursorWait = 8;
					
						if (mSelectedIndex < 0) {
							mSelectedIndex = mItemList.length - 1;
						}
				
						break;
					
					case SoftPad.PADACTIVE_DOWN:
					
						mSelectedIndex++;
						mCursorWait = 8;
					
						if (mSelectedIndex >= mItemList.length) {
							mSelectedIndex = 0;
						}

						break;

					}
				}

			}

			break;

		// ページ終了処理
		case SelectDialog.PROCESS_STAGE_SELECTED:

			// リターンキー解放待機後にページ表示処理開始
			if (pad.getTrigger() == 0) {

				mProcessStage = SelectDialog.PROCESS_STAGE_EXIT;

			}

			break;

		// メッセージ（全ページ）終了処理
		case SelectDialog.PROCESS_STAGE_EXIT:

			break;

		}

		mProcessCount++;

		// 表示終了状態ならtrue
		return mProcessStage == SelectDialog.PROCESS_STAGE_EXIT;

	}

	public void draw(Canvas cv) {

		if (!mVisible || mItemList == null || mItemList.length < 1) {
			return;
		}

		cv.drawRect(mWindowRect, mBorderPaint);
		cv.drawRect(mClientRect, mBgPaint);

		for (int i = 0;i < mItemList.length;i++) {
			
			int lineCenter = mContentRect.top + i * mLineHeight + (int)(mLineHeight / 2);
			
			cv.drawText(mItemList[i], mStringItemRect.left, lineCenter - ((mTextPaint.ascent() + mTextPaint.descent()) / 2), mTextPaint);

		}

		cv.drawPath(mSelectMarkerPathList[mSelectedIndex], mSelectMarkerPaint);

	}

}
