package jp.programmingmat.www.atfuiwnd;

import android.view.View;
import android.view.MotionEvent;
import android.content.Context;
import android.graphics.*;

/*
	タッチ操作ソフトパッド
*/
public class SoftPad extends View {

	private Paint mBGPaint = null;
	private Paint mActivePaint = null;

	private Paint mStickBGOffPaint, mStickLineOffPaint;
	private Paint mStickBGOnPaint, mStickLineOnPaint;
	
	public int mPadActive = 0;

	private static final int KEYIDNUM = 5;
	private static final int KEYID_UP = 0;
	private static final int KEYID_RIGHT = 1;
	private static final int KEYID_DOWN = 2;
	private static final int KEYID_LEFT = 3;
	private static final int KEYID_TRIGGER_A = 4;
	
	public static final int PADACTIVE_UP = 1;
	public static final int PADACTIVE_RIGHT = 2;
	public static final int PADACTIVE_DOWN = 4;
	public static final int PADACTIVE_LEFT = 8;
	public static final int PADACTIVE_TRIGGER_A = 16;

	public static final int STICK_MASK = 0xf;
	public static final int TRIGGER_MASK = 0x10;

	private Path[] mKeyPathList;
	private Region[] mKeyRegionList;

	private Region mPadArea;

	public SoftPad(Context context) {

		super(context);

		// クリック受け入れ設定
		setClickable(true);

		// 背景色を設定
		mBGPaint = new Paint();
		mBGPaint.setColor(0xffcccccc);

		// 押下状態の塗りつぶし色を設定
		mActivePaint = new Paint();
		mActivePaint.setColor(0xff808080);

		mStickBGOffPaint = new Paint();
		mStickBGOffPaint.setColor(0xffc0c0c0);
		
		mStickLineOffPaint = new Paint();
		mStickLineOffPaint.setColor(0xff408080);
		mStickLineOffPaint.setStyle(Paint.Style.STROKE);
	
		mStickBGOnPaint = new Paint();
		mStickBGOnPaint.setColor(0xff808080);

		mStickLineOnPaint = new Paint();
		mStickLineOnPaint.setColor(0xff40c0ff);
		mStickLineOnPaint.setStyle(Paint.Style.STROKE);
	
		mKeyPathList = new Path[KEYIDNUM];

		for (int i = 0;i < KEYIDNUM;i++) {
			mKeyPathList[i] = new Path();
		}

		mKeyRegionList = new Region[KEYIDNUM];

		for (int i = 0;i < KEYIDNUM;i++) {
			mKeyRegionList[i] = new Region();
		}
		
	}

	public int getStick() {
		return mPadActive & STICK_MASK;
	}

	public int getTrigger() {
		return mPadActive & TRIGGER_MASK;
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent e) {

		// タッチ時の状態に応じて処理
		switch(e.getAction()) {

		// タッチ状態
		case MotionEvent.ACTION_DOWN:
		case MotionEvent.ACTION_MOVE:

			mPadActive = 0;
			
			for (int i = 0;i < KEYIDNUM;i++) {
				if (mKeyRegionList[i].contains((int)(e.getX()), (int)(e.getY()))) {
					mPadActive |= (int)(Math.pow(2, i));
				}
			}

			invalidate();

			return true;

		// タッチ終了
		case MotionEvent.ACTION_UP:
		case MotionEvent.ACTION_CANCEL:

			// 押下状態を解除
			mPadActive = 0;

			invalidate();

			return true;

		}

		return false;

	}

	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {

		int stickLeft = 0;
		int stickTop = 0;
		int stickRight = 0;
		int stickBottom = 0;
		
		int triggerLeft = 0;
		int triggerTop = 0;
		int triggerRight = 0;
		int triggerBottom = 0;

		int triggerSize = 0;
		
		int stickSpace = 4;

		// 縦／横状態に応じて方向キーとトリガーボタンの領域を決定
		if (w > h) {
			
			stickRight = h;
			stickBottom = h;

			if (h > (w - stickRight)) {
				triggerSize = w - stickRight;
			} else {
				triggerSize = h;
			}

			triggerSize = (int)(triggerSize * 0.7);
			
			triggerRight = w;
			triggerLeft = triggerRight - triggerSize;

			triggerTop = (stickTop + stickBottom) / 2  - (triggerSize / 2);
			triggerBottom = triggerTop + triggerSize;
			
		} else {
			
			stickRight = w;
			stickBottom = w;

			if (w > (h - w)) {
				triggerSize = h - w;
			} else {
				triggerSize = w;
			}

			triggerSize = (int)(triggerSize * 0.7);

			triggerBottom = h;
			triggerTop = triggerBottom - triggerSize;
			
			triggerLeft = (stickLeft + stickRight) / 2 - (triggerSize / 2);
			triggerRight = triggerLeft + triggerSize;
			
		}

		int stickWidth = stickRight - stickLeft + 1;
		int stickHeight = stickBottom - stickTop + 1;
		int stickCenterX = (stickLeft + stickRight) / 2;
		int stickCenterY = (stickTop + stickBottom) / 2;
		
		mPadArea = new Region(0, 0, w, h);
		
		// 上キー用パスを作成
		mKeyPathList[KEYID_UP].moveTo(stickCenterX - stickWidth / 12, stickTop);
		mKeyPathList[KEYID_UP].lineTo(stickCenterX - stickWidth / 6, stickTop + stickHeight / 5);
		mKeyPathList[KEYID_UP].lineTo((float)(stickCenterX - stickWidth / 6.5), stickTop + stickHeight / 3 - stickSpace);
		mKeyPathList[KEYID_UP].lineTo((float)(stickCenterX + stickWidth / 6.5), stickTop + stickHeight / 3 - stickSpace);
		mKeyPathList[KEYID_UP].lineTo(stickCenterX + stickWidth / 6, stickTop + stickHeight / 5);
		mKeyPathList[KEYID_UP].lineTo(stickCenterX + stickWidth / 12,  stickTop);
		mKeyPathList[KEYID_UP].close();

		// 右キー用パスを作成
		mKeyPathList[KEYID_RIGHT] = new Path();
		mKeyPathList[KEYID_RIGHT].moveTo(stickRight, stickCenterY - stickHeight / 12);
		mKeyPathList[KEYID_RIGHT].lineTo(stickRight - stickWidth / 5, stickCenterY - stickHeight / 6);
		mKeyPathList[KEYID_RIGHT].lineTo(stickRight - stickWidth / 3 + stickSpace, (float)(stickCenterY - stickHeight / 6.5));
		mKeyPathList[KEYID_RIGHT].lineTo(stickRight - stickWidth / 3 + stickSpace, (float)(stickCenterY + stickHeight / 6.5));
		mKeyPathList[KEYID_RIGHT].lineTo(stickRight - stickWidth / 5, stickCenterX + stickHeight / 6);
		mKeyPathList[KEYID_RIGHT].lineTo(stickRight, stickCenterY + stickHeight / 12);
		mKeyPathList[KEYID_RIGHT].close();

		// 下キー用パスを作成
		mKeyPathList[KEYID_DOWN].moveTo(stickCenterX - stickWidth / 12, stickBottom);
		mKeyPathList[KEYID_DOWN].lineTo(stickCenterX - stickWidth / 6, stickBottom - stickHeight / 5);
		mKeyPathList[KEYID_DOWN].lineTo((float)(stickCenterX - stickWidth / 6.5), stickBottom - stickHeight / 3 + stickSpace);
		mKeyPathList[KEYID_DOWN].lineTo((float)(stickCenterX + stickWidth / 6.5), stickBottom - stickHeight / 3 + stickSpace);
		mKeyPathList[KEYID_DOWN].lineTo(stickCenterX + stickWidth / 6, stickBottom - stickHeight / 5);
		mKeyPathList[KEYID_DOWN].lineTo(stickCenterX + stickWidth / 12,  stickBottom);
		mKeyPathList[KEYID_DOWN].close();

		// 右キー用パスを作成
		mKeyPathList[KEYID_LEFT].moveTo(stickLeft, stickCenterY - stickHeight / 12);
		mKeyPathList[KEYID_LEFT].lineTo(stickLeft + stickWidth / 5, stickCenterY - stickHeight / 6);
		mKeyPathList[KEYID_LEFT].lineTo(stickLeft + stickWidth / 3 - stickSpace, (float)(stickCenterY - stickHeight / 6.5));
		mKeyPathList[KEYID_LEFT].lineTo(stickLeft + stickWidth / 3 - stickSpace, (float)(stickCenterY + stickHeight / 6.5));
		mKeyPathList[KEYID_LEFT].lineTo(stickLeft + stickWidth / 5, stickCenterX + stickHeight / 6);
		mKeyPathList[KEYID_LEFT].lineTo(stickLeft, stickCenterY + stickHeight / 12);
		mKeyPathList[KEYID_LEFT].close();

		// ボタン用パスを作成
		mKeyPathList[KEYID_TRIGGER_A].addCircle((int)((triggerLeft + triggerRight) / 2), (int)((triggerTop + triggerBottom) / 2), (int)(triggerSize / 2), Path.Direction.CW);
		
		for (int i = 0;i < KEYIDNUM;i++) {
			mKeyRegionList[i].setPath(mKeyPathList[i], mPadArea);
		}
		
	}

	@Override
	protected void onDraw(Canvas cv) {

		for (int i = 0;i < KEYIDNUM;i++) {

			// 現在の押下状態に応じて各キーを描画
			if ((mPadActive & (int)(Math.pow(2, i))) != 0) {

				cv.drawPath(mKeyPathList[i], mStickBGOnPaint);
				cv.drawPath(mKeyPathList[i], mStickLineOnPaint);

			} else {

				cv.drawPath(mKeyPathList[i], mStickBGOffPaint);
				cv.drawPath(mKeyPathList[i], mStickLineOffPaint);

			}

		}
		
	}

}