package jp.programmingmat.www.atfuiwnd;

import android.content.Context;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Bitmap;
import android.graphics.Rect;

/*
	マップ描画ビュー
*/
public class appView extends SurfaceView implements SurfaceHolder.Callback, Runnable {

	private Bitmap mBmpChips, mBmpChar;

	private int mChipWidth, mChipHeight;
	private int mChipCols, mChipRows;
	private int mMapCols, mMapRows;

	private int mX, mY;
	private int mVX, mVY;
	private int mVDx, mVDy;

	private int mScrollDir, mScrollPos, mScrollPixel;

	private int[] mMapData;

	private Thread thread;
	private SurfaceHolder _holder;

	private volatile boolean _alive;

	private SoftPad mPad;

	private long process_default_ns = 10 * 1000000;

	private long frame_ns = 0;
	private long frame_ns_sum = 0;
	private long frame_ms_disp = 0;

	private final int GAMESTATE_WALK = 0;
	private final int GAMESTATE_TEST = 1;
	
	private int mTestStage;
	
	private int mGameState;
	
	private Paint mDispPaint;

	MessageWindow mMesWnd;
	SelectDialog mSelectDlg;
	
	public appView(Context context, SoftPad pad) {

		super(context);

		_holder = getHolder();

		_holder.addCallback(this);

		mPad = pad;

		mGameState = GAMESTATE_WALK;
		
		// 処理時間表示用paint作成
		mDispPaint = new Paint();

		mDispPaint.setTextSize(18);

		mMesWnd = new MessageWindow();
		mMesWnd.setText("ページ1テスト文字列<:br>改行。以下改ページ挿入<:page>page2<:br><:br>改行二つ挿入。<:br>長い文字列はウインドウ枠内で折り返されます。ページ領域を超えると、ボタン入力を待って次ページに移動する自動改ページが入ります。");

		mSelectDlg = new SelectDialog();
		mSelectDlg.setItems(new String[]{"選択肢1", "選択肢2"});
		
		mTestStage = 0;
		
	}

	public void setChipBitmap(Bitmap bmp, int cwidth, int cheight) {

		mBmpChips = Bitmap.createBitmap(bmp);

		mChipWidth = cwidth;
		mChipHeight = cheight;

		// 画面上に表示するマップチップ数を設定
		mChipCols = 11;
		mChipRows = 11;

		// マップサイズ設定
		mMapCols = 30;
		mMapRows = 30;

		// マップデータ配列を作成
		mMapData = new int[mMapCols * mMapRows];

		// マップデータを作成
		for (int i = 0;i < mMapData.length;i++) {
			mMapData[i] = (int)(Math.random() * 16);
		}

		// マップ上の初期位置設定
		mX = 10;
		mY = 10;

		mVX = mX;
		mVY = mY;

		mVDx = 0;
		mVDy = 0;

		// スクロール単位を1ピクセルに設定
		mScrollPixel = 1;

	}

	public void setCharBitmap(Bitmap bmp) {
		mBmpChar = bmp;
	}
	
	public void surfaceChanged (SurfaceHolder holder, int format, int width, int height) {
	}

	public void surfaceCreated(SurfaceHolder holder) {

		// メインループ実行フラグをセット
		_alive = true;

		// メインループ用スレッド作成
		thread = new Thread(this);
		thread.start();

	}

	public void surfaceDestroyed(SurfaceHolder holder) {

		// メインループ実行フラグをリセット
		_alive = false;

	}

	public void run() {

		int process_count = 0;
		long process_prev_ns = 0;
		long process_ns = 0;

		long frame_prev_ns = 0;

		// メインループ
		while (_alive) {
		
			// 今回の処理開始時刻を取得
			frame_ns = System.nanoTime();

			// 前回処理からの時間を処理時間累計に加算
			frame_ns_sum += frame_ns - frame_prev_ns;

			// 今回の処理開始時刻を記録
			frame_prev_ns = frame_ns;

			// 10回に一度表示用の処理時間を更新
			if (process_count % 10 == 9) {

				// 処理時間の平均をミリ秒単位で算出
				frame_ms_disp = (long)((frame_ns_sum / 10) / 1000000);

				// 処理時間累計をクリア
				frame_ns_sum = 0;

			}

			while (process());

			switch ((process_count / 5) % 3) {

			case 0:

				mDispPaint.setColor(0xff800000);

				break;

			case 1:

				mDispPaint.setColor(0xff008000);

				break;

			case 2:

				mDispPaint.setColor(0xff000080);

				break;

			}

			Canvas cv = _holder.lockCanvas();
			onDraw(cv);
			_holder.unlockCanvasAndPost(cv);

			// 前回の処理終了時からの経過時間を算出
			process_ns = System.nanoTime() - process_prev_ns;

			// ウエイトを算出
			long wait_ns = process_default_ns - process_ns;

			long wait_ms = wait_ns / 1000000;

			if (wait_ms < 1) {
				wait_ms = 1;
			}

			// ウエイト
			try {
				Thread.sleep(wait_ms);
			} catch (Exception e) {
				if (!_alive) {
					break;
				}
			}

			// 今回の処理終了時間を記録
			process_prev_ns = System.nanoTime();

			process_count++;

		}

	}

	private boolean process() {

		boolean result = false;
		
		switch (mGameState) {

		case GAMESTATE_WALK:
			
			result = walk();
			
			break;

		case GAMESTATE_TEST:

			test();

			break;
			
		}

		return result;
		
	}
	
	private boolean walk() {
		
		boolean result = false;
		
		if (mScrollDir == 0) { // 非スクロール時

			// パッドがタッチされていたらスクロール開始
			switch (mPad.getStick()) {

			case SoftPad.PADACTIVE_UP:

				if (mY > 0) {

					mScrollDir = mPad.getStick();
					mScrollPos = 0;

					mVX = mX;
					mVY = mY - 1;

					mVDx = 0;
					mVDy = -mChipHeight;

				}

				break;

			case SoftPad.PADACTIVE_RIGHT:

				if (mX < (mMapCols - mChipCols)) {

					mScrollDir = mPad.getStick();
					mScrollPos = 0;

					mVX = mX;
					mVY = mY;

					mVDx = 0;
					mVDy = 0;

				}

				break;

			case SoftPad.PADACTIVE_DOWN:

				if (mY < (mMapRows - mChipRows)) {

					mScrollDir = mPad.getStick();
					mScrollPos = 0;

					mVX = mX;
					mVY = mY;

					mVDx = 0;
					mVDy = 0;

				}

				break;

			case SoftPad.PADACTIVE_LEFT:

				if (mX > 0) {

					mScrollDir = mPad.getStick();
					mScrollPos = 0;

					mVX = mX - 1;
					mVY = mY;

					mVDx = -mChipWidth;
					mVDy = 0;

				}

				break;

			}

		} else { // スクロール中

			switch (mScrollDir) {

			case SoftPad.PADACTIVE_UP:

				mVDy = -mChipHeight + mScrollPos;

				mScrollPos += mScrollPixel;

				if (mScrollPos >= mChipHeight) {

					mY--;

					mVX = mX;
					mVY = mY;

					mVDx = 0;
					mVDy = 0;

					mScrollDir = 0;

					if (mPad.getStick() ==SoftPad.PADACTIVE_UP) {
						result = true;
					} else {
						if (mTestStage < 2) {
							
							mTestStage++;

							mGameState = GAMESTATE_TEST;
							
						}
					}
					
				}

				break;

			case SoftPad.PADACTIVE_RIGHT:

				mVDx = -mScrollPos;

				mScrollPos += mScrollPixel;

				if (mScrollPos >= mChipWidth) {

					mX++;

					mVX = mX;
					mVY = mY;

					mVDx = 0;
					mVDy = 0;

					mScrollDir = 0;

					if (mPad.getStick() ==SoftPad.PADACTIVE_RIGHT) {
						result = true;
					} else {
						if (mTestStage < 2) {

							mTestStage++;

							mGameState = GAMESTATE_TEST;
							
						}
					}
					
				}

				break;

			case SoftPad.PADACTIVE_DOWN:

				mVDy = -mScrollPos;

				mScrollPos += mScrollPixel;

				if (mScrollPos >= mChipHeight) {

					mY++;

					mVX = mX;
					mVY = mY;

					mVDx = 0;
					mVDy = 0;

					mScrollDir = 0;

					if (mPad.getStick() ==SoftPad.PADACTIVE_DOWN) {
						result = true;
					} else {
						if (mTestStage < 2) {
							
							mTestStage++;

							mGameState = GAMESTATE_TEST;
							
						}
					}
					
				}

				break;

			case SoftPad.PADACTIVE_LEFT:

				mVDx = -mChipWidth + mScrollPos;

				mScrollPos += mScrollPixel;

				if (mScrollPos >= mChipWidth) {

					mX--;

					mVX = mX;
					mVY = mY;

					mVDx = 0;
					mVDy = 0;

					mScrollDir = 0;

					if (mPad.getStick() ==SoftPad.PADACTIVE_LEFT) {
						result = true;
					} else {
						if (mTestStage < 2) {

							mTestStage++;

							mGameState = GAMESTATE_TEST;
							
						}
					}
					
				}

				break;

			}

		}

		return result;
		
	}

	public void test() {
	
		switch (mTestStage) {
		
		case 1:
			
			mMesWnd.setVisible(true);

			if (mMesWnd.process(mPad)) {
			
				mMesWnd.setVisible(false);
				mGameState = GAMESTATE_WALK;

			}
			
			break;
			
		case 2:

			mSelectDlg.setVisible(true);

			if (mSelectDlg.process(mPad)) {
			
				mSelectDlg.setVisible(false);
				mGameState = GAMESTATE_WALK;

			}
			
			break;
			
		}

	}

	@Override
	protected void onDraw(Canvas cv) {

		Rect src = new Rect();
		Rect dst = new Rect();

		// マップ画面を描画
		for (int i = 0; i < mChipRows + 1;i++) {
			for (int j = 0; j < mChipCols + 1;j++) {

				// 描画位置に対応するマップチップを取得
				int tip = mMapData[(j + mVX) + (i + mVY) * mChipCols];

				// チップ画像上の位置を算出
				int chip_x = tip % 4;
				int chip_y = (int)(Math.floor(tip / 4));

				// チップ画像内の転送位置を算出
				src.left = chip_x * mChipWidth;
				src.top = chip_y * mChipHeight;
				src.right = src.left + mChipWidth;
				src.bottom = src.top + mChipHeight;

				// 描画範囲を算出
				dst.left = j * mChipWidth + mVDx;
				dst.top = i * mChipHeight + mVDy;
				dst.right = dst.left + mChipWidth;
				dst.bottom = dst.top + mChipHeight;

				// チップ画像を転送
				cv.drawBitmap(mBmpChips, src, dst, null);

			}
		}

		src.left = 0 * mChipWidth;
		src.top = 4 * mChipHeight;
		src.right = src.left + mChipWidth;
		src.bottom = src.top + mChipHeight;
	
		dst.top = mChipHeight * 5;
		dst.left = mChipWidth * 5;
		dst.right = dst.left + mChipWidth;
		dst.bottom = dst.top + mChipHeight;

		cv.drawBitmap(mBmpChar, src, dst, null);
	
		cv.drawText("フレーム処理平均時間:" + frame_ms_disp + "ms", 0, mDispPaint.descent() - mDispPaint.ascent(), mDispPaint);

		mMesWnd.draw(cv);
		mSelectDlg.draw(cv);
		
	}

}